@@ -26,4 +26,12 @@
 
 #define NV_PGC6_AON_FRTS_INPUT_WPR_SIZE_SECURE_SCRATCH_GROUP_03_0_WPR_SIZE_1MB_IN_4K                        0x100
 
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC                      NV_PGC6_AON_SECURE_SCRATCH_GROUP_20
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_MODE_ENABLED         0:0
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_MODE_ENABLED_TRUE    0x1
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_MODE_ENABLED_FALSE   0x0
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_DEV_ENABLED          1:1
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_DEV_ENABLED_TRUE     0x1
+#define NV_PGC6_AON_SECURE_SCRATCH_GROUP_20_CC_DEV_ENABLED_FALSE    0x0
+
 #endif // __gh100_dev_gc6_island_addendum_h__
