@@ -29,6 +29,7 @@
 #include "gpu/conf_compute/conf_compute.h"
 #include "gpu/fsp/kern_fsp.h"
 #include "gpu/gsp/kernel_gsp.h"
+#include "gpu/mem_sys/kern_mem_sys.h"
 #include "gsp/gspifpub.h"
 #include "vgpu/rpc.h"
 
@@ -296,8 +297,6 @@ kgspCalculateFbLayout_GH100
     NV_ASSERT_OR_RETURN(pKernelGsp->gspRmBootUcodeSize != 0, NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pRiscvDesc != NULL, NV_ERR_INVALID_STATE);
 
-    portMemSet(pWprMeta, 0, sizeof *pWprMeta);
-
     //
     // We send this to FSP as the size to reserve above FRTS.
     // The actual offset gets filled in by ACR ucode when it sets up WPR2.
@@ -345,6 +344,16 @@ kgspCalculateFbLayout_GH100
     pWprMeta->gspFwHeapVfPartitionCount =
         pGpu->bVgpuGspPluginOffloadEnabled ? MAX_PARTITIONS_WITH_GFID : 0;
 
+    // CrashCat queue (if allocated in sysmem)
+    KernelCrashCatEngine *pKernelCrashCatEng = staticCast(pKernelGsp, KernelCrashCatEngine);
+    MEMORY_DESCRIPTOR *pCrashCatQueueMemDesc = kcrashcatEngineGetQueueMemDesc(pKernelCrashCatEng);
+    if (pCrashCatQueueMemDesc != NULL)
+    {
+        NV_ASSERT_CHECKED(memdescGetAddressSpace(pCrashCatQueueMemDesc) == ADDR_SYSMEM);
+        pWprMeta->sysmemAddrOfCrashReportQueue = memdescGetPhysAddr(pCrashCatQueueMemDesc, AT_GPU, 0);
+        pWprMeta->sizeOfCrashReportQueue = (NvU32)memdescGetSize(pCrashCatQueueMemDesc);
+    }
+
     // Fill in the meta-metadata
     pWprMeta->revision = GSP_FW_WPR_META_REVISION;
     pWprMeta->magic = GSP_FW_WPR_META_MAGIC;
@@ -403,27 +412,32 @@ kgspSetupGspFmcArgs_GH100
 }
 
 /*!
- * Determine if lockdown is released.
+ * Determine if PRIV lockdown is released or the FMC has encountered an error.
  */
 static NvBool
-_kgspIsLockdownReleased
+_kgspLockdownReleasedOrFmcError
 (
     OBJGPU  *pGpu,
     void    *pVoid
 )
 {
     KernelGsp *pKernelGsp = reinterpretCast(pVoid, KernelGsp *);
-    NvU32 reg;
+    KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
+    NvU32 hwcfg2, mailbox0;
 
-    reg = kflcnRegRead_HAL(pGpu, staticCast(pKernelGsp, KernelFalcon), NV_PFALCON_FALCON_HWCFG2);
+    //
+    // If lockdown has not been released, check NV_PGSP_FALCON_MAILBOX0, where the GSP-FMC
+    // (namely ACR) logs error codes during boot. GSP-FMC reported errors are always fatal,
+    // so there's no reason to continue polling for lockdown release.
+    //
+    mailbox0 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0);
+    hwcfg2 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_HWCFG2);
 
-    return FLD_TEST_DRF(_PFALCON, _FALCON_HWCFG2, _RISCV_BR_PRIV_LOCKDOWN,
-                        _UNLOCK, reg);
+    return (FLD_TEST_DRF(_PFALCON, _FALCON_HWCFG2, _RISCV_BR_PRIV_LOCKDOWN, _UNLOCK, hwcfg2) ||
+            (mailbox0 != 0));
 }
 
 
-
-
 static void
 _kgspBootstrapGspFmc_GH100
 (
@@ -514,6 +528,7 @@ kgspBootstrapRiscvOSEarly_GH100
     KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
     KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
     NV_STATUS     status        = NV_OK;
+    NvU32         mailbox0;
 
     // Only for GSP client builds
     if (!IS_GSP_CLIENT(pGpu))
@@ -523,7 +538,7 @@ kgspBootstrapRiscvOSEarly_GH100
     }
 
     // Setup the descriptors that GSP-FMC needs to boot GSP-RM
-    NV_ASSERT_OK_OR_RETURN(kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw));
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw));
 
     kgspSetupLibosInitArgs(pGpu, pKernelGsp);
 
@@ -538,21 +553,21 @@ kgspBootstrapRiscvOSEarly_GH100
     if (status != NV_OK)
     {
         NV_ASSERT_OK_FAILED("NV_RM_RPC_GSP_SET_SYSTEM_INFO", status);
-        goto exit;
+        return status;
     }
 
     NV_RM_RPC_SET_REGISTRY(pGpu, status);
     if (status != NV_OK)
     {
         NV_ASSERT_OK_FAILED("NV_RM_RPC_SET_REGISTRY", status);
-        goto exit;
+        return status;
     }
 
     if (pKernelFsp != NULL && !pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_GSPFMC))
     {
         NV_PRINTF(LEVEL_NOTICE, "Starting to boot GSP via FSP.\n");
         pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_GSP_MODE_GSPRM, NV_TRUE);
-        NV_ASSERT_OK_OR_RETURN(kfspSendBootCommands_HAL(pGpu, pKernelFsp));
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, kfspSendBootCommands_HAL(pGpu, pKernelFsp));
     }
     else
     {
@@ -579,8 +594,8 @@ kgspBootstrapRiscvOSEarly_GH100
         }
     }
 
-    // Wait for lockdown to be released.
-    status = gpuTimeoutCondWait(pGpu, _kgspIsLockdownReleased, pKernelGsp, NULL);
+    // Wait for lockdown to be released or the FMC to report an error
+    status = gpuTimeoutCondWait(pGpu, _kgspLockdownReleasedOrFmcError, pKernelGsp, NULL);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Timeout waiting for lockdown release. It's also "
@@ -598,6 +613,12 @@ kgspBootstrapRiscvOSEarly_GH100
                   kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1));
         return status;
     }
+    else if ((mailbox0 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0)) != 0)
+    {
+        NV_PRINTF(LEVEL_ERROR, "GSP-FMC reported an error while attempting to boot GSP: 0x%x\n",
+                  mailbox0);
+        return NV_ERR_NOT_READY;
+    }
 
     // Start polling for libos logs now that lockdown is released
     pKernelGsp->bLibosLogsPollingEnabled = NV_TRUE;
@@ -614,25 +635,19 @@ kgspBootstrapRiscvOSEarly_GH100
     else
     {
         NV_ASSERT_FAILED("Failed to boot GSP");
-        status = NV_ERR_NOT_READY;
-        goto exit;
+        return NV_ERR_NOT_READY;
     }
 
     NV_PRINTF(LEVEL_INFO, "Waiting for GSP fw RM to be ready...\n");
 
     // Link the status queue.
-    NV_ASSERT_OK_OR_GOTO(status, GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo),
-                          exit);
+    NV_ASSERT_OK_OR_RETURN(GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo));
 
-    NV_ASSERT_OK_OR_GOTO(status, kgspWaitForRmInitDone(pGpu, pKernelGsp),
-                          exit);
+    NV_ASSERT_OK_OR_RETURN(kgspWaitForRmInitDone(pGpu, pKernelGsp));
 
     NV_PRINTF(LEVEL_INFO, "GSP FW RM ready.\n");
 
-exit:
-    NV_ASSERT(status == NV_OK);
-
-    return status;
+    return NV_OK;
 }
 
 void
